/**
 * Copyright 2012 52North Initiative for Geospatial Open Source Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.n52.android.tracker;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Random;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;
import org.n52.android.tracker.Constants.ValueTypes;
import org.n52.android.tracker.datastructure.Point;

import android.text.format.Time;
import android.util.Log;

public class Utils {

	private static final String AB = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	private static Random rnd = new Random();

	/**
	 * Converts inputstream to string
	 * 
	 * @param is
	 * @return
	 * @throws IOException
	 */
	public static String convertStreamToString(InputStream is) throws IOException {
		if (is != null) {
			Writer writer = new StringWriter();
			char[] buffer = new char[1024];
			try {
				Reader reader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
				int n;
				while ((n = reader.read(buffer)) != -1) {
					writer.write(buffer, 0, n);
				}
			} finally {
				is.close();
			}
			return writer.toString();
		} else {
			return "";
		}
	}

	public static String getStringTime(long time) {
		Time timeFormat = new Time();
		timeFormat.set(time);
		return "" + timeFormat.format("%d-%m-%Y %H:%M:%S");
	}

	public static double round(double number, int i) {
		double faktor = Math.pow(10, i);
		number = number * faktor;
		number = Math.round(number);
		number = number / faktor;
		return number;
	}

	public static String getSosConformTimeString(long time) {
		Time timeFormat = new Time();
		timeFormat.set(time);
		return "" + timeFormat.format3339(false);
	}

	public static String getSosConformPointString(Point point) {
		return "POINT(" + point.getLongitude() + " " + point.getLatitude() + ")#4326";
	}

	public static Object generateRandomValue(ValueTypes valueTypes) {
		if (valueTypes.equals(Constants.ValueTypes.textType)) {
			return randomString(8);
		} else if (valueTypes.equals(Constants.ValueTypes.numericType)) {
			return new Random().nextDouble();
		} else if (valueTypes.equals(Constants.ValueTypes.booleanType)) {
			return new Random().nextBoolean();
		} else if (valueTypes.equals(Constants.ValueTypes.countType)) {
			return new Random().nextInt();
		}
		return null;
	}

	private static String randomString(int len) {
		StringBuilder sb = new StringBuilder(len);
		for (int i = 0; i < len; i++)
			sb.append(AB.charAt(rnd.nextInt(AB.length())));
		return sb.toString();
	}

	/**
	 * 
	 * Upload a file to a FTP server with the following syntax:
	 * ftp://user:password@host:port/filePath;type=i.
	 * 
	 * @throws IOException
	 */
	public static void ftpUpload(String server,
			String user,
			String password,
			String resourcePathAndName,
			String fileContent) {
		FTPClient ftp = new FTPClient();

		try {
			int reply;
			ftp.connect(server);
			ftp.user(user);
			ftp.pass(password);
			ftp.setFileType(FTP.ASCII_FILE_TYPE);

			Log.i("FTPUpload", "Connected to " + server + ":" + ftp.getReplyString());

			// check the reply code to verify success
			reply = ftp.getReplyCode();

			if (!FTPReply.isPositiveCompletion(reply)) {
				ftp.disconnect();
				Log.i("FTPUpload", "FTP server refused connection.");
				return;
			}

			// transfer files
			ByteArrayInputStream bais = new ByteArrayInputStream(fileContent.getBytes());
			ftp.enterLocalPassiveMode();
			boolean storeFile = ftp.storeFile(resourcePathAndName, bais);
			if (storeFile)
				Log.i("FTPUpload", "Uploaded file: " + ftp.getReplyString());
			else
				Log.i("FTPUpload", "Storing file returned error: " + ftp.getReplyCode());
			ftp.logout();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (ftp.isConnected()) {
				try {
					ftp.disconnect();
				} catch (IOException ioe) {
					// do nothing
				}
			}
		}
	}

	public static Point generateRandomNearPoint(Point point) {
		double longitude = point.getLongitude() + (new Random().nextDouble() / 1000);
		double latitude = point.getLatitude() + (new Random().nextDouble() / 1000);
		return new Point(longitude, latitude, point.getAltitude());
	}

}
