/**
 * Copyright 2012 52North Initiative for Geospatial Open Source Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.n52.android.tracker.activities;

import java.util.List;

import org.n52.android.tracker.R;
import org.n52.android.tracker.encoder.Templates;
import org.n52.android.tracker.services.impl.TrackerService;
import org.n52.android.tracker.services.impl.TrackerService.TrackerServiceBinder;

import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.location.LocationManager;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.MyLocationOverlay;

/**
 * This activity shows the actual location on a map and is the entry point for the app.
 * 
 * @author Jan Schulte
 * 
 */
public class Tracker extends MapActivity {

	@SuppressWarnings("unused")
	private TrackerServiceBinder trackerServBinder;

	private Intent trackerIntent;

	private MapView mapView;
	private MapController mapCon;
	private MyLocationOverlay locOverlay;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		initService();

		initMapView();

		initLocationOverlay();
	}

	private void initLocationOverlay() {
		locOverlay = new MyLocationOverlay(this, mapView) {

			public synchronized void onLocationChanged(Location location) {
				super.onLocationChanged(location);
				final GeoPoint ownLocation = new GeoPoint((int) (location.getLatitude() * 1E6),
						(int) (location.getLongitude() * 1E6));
				if (ownLocation != null) {
					mapCon.animateTo(ownLocation);
				}
			}
		};

		mapView.getOverlays().add(locOverlay);
		locOverlay.enableMyLocation();
		locOverlay.enableCompass();

		locOverlay.runOnFirstFix(new Runnable() {
			public void run() {
				mapCon.animateTo(locOverlay.getMyLocation());
			}
		});
	}

	private void initService() {
		Logging.addEntry("Start Logging");
		// init location
		Data.lastLoc = getLatestLocation();
		// init templates
		Templates.initTemplates(this);
		// start tracker service
		trackerIntent = new Intent(this, TrackerService.class);
		startService(trackerIntent);
	}

	private Location getLatestLocation() {
		float bestAccuracy = Float.MAX_VALUE;
		LocationManager locationManager = (LocationManager) this.getSystemService(Context.LOCATION_SERVICE);
		List<String> matchingProviders = locationManager.getAllProviders();
		Location bestResult = null;
		for (String provider : matchingProviders) {
			Location location = locationManager.getLastKnownLocation(provider);
			if (location != null) {
				float accuracy = location.getAccuracy();
				if (accuracy < bestAccuracy) {
					bestResult = location;
					bestAccuracy = accuracy;
				}
			}
		}
		return bestResult;
	}

	private void initMapView() {
		mapView = (MapView) findViewById(R.id.mapview_karte);
		mapCon = mapView.getController();

		final int maxZoomlevel = mapView.getMaxZoomLevel();
		mapCon.setZoom(maxZoomlevel - 4);

		mapView.setBuiltInZoomControls(true);

	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.hauptmenu, menu);
		return super.onCreateOptionsMenu(menu);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menu_data:
			startActivity(new Intent(this, Data.class));
			return true;
		case R.id.menu_logs:
			startActivity(new Intent(this, Logging.class));
			return true;
		case R.id.menu_preferences:
			startActivity(new Intent(this, Preferences.class));
			return true;
		case R.id.menu_start:
			startActivity(new Intent(this, StartRecord.class));
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}

	@Override
	protected void onResume() {
		locOverlay.enableMyLocation();
		super.onResume();
	}

	@Override
	protected void onPause() {
		if (isFinishing() && !TrackerService.tracking) {
			stopService(trackerIntent);
		}
		locOverlay.disableMyLocation();
		super.onPause();
	}

	@Override
	protected boolean isRouteDisplayed() {
		return false;
	}
}