/**
 * Copyright 2012 52North Initiative for Geospatial Open Source Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.n52.android.tracker.connection;

import org.jivesoftware.smack.SASLAuthentication;
import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.XMPPException;
import org.jivesoftware.smackx.muc.MultiUserChat;
import org.n52.android.tracker.Configuration;
import org.n52.android.tracker.Utils;
import org.n52.android.tracker.Constants.ValueTypes;
import org.n52.android.tracker.datastructure.Observation;
import org.n52.android.tracker.datastructure.Phenomenon;
import org.n52.android.tracker.datastructure.Sensor;
import org.n52.android.tracker.encoder.RequestBuilder;
import org.n52.android.tracker.sensorbus.ConnectSensorMessage;
import org.n52.android.tracker.sensorbus.IMessage;
import org.n52.android.tracker.sensorbus.PublishDataMessage;

import android.util.Log;

/**
 * @author Jan Schulte
 * 
 */
public class SensorBusConnection implements IConnection {

	private static final String TAG = "SensorBusConnection";

	private XMPPConnection conn;

	private String hostname;

	private String login;

	private String passwort;

	private String channel;

	private MultiUserChat chat;

	public SensorBusConnection(String hostname, String login, String passwort, String channel) {
		this.hostname = hostname;
		this.login = login;
		this.passwort = passwort;
		this.channel = channel;
	}

	public void registerSensor(Sensor sensor) {
		try {
			// connect to sensorBus
			connect();
			sensor.setOfferingDescription(sensor.getOfferingDescription() + " by SensorBus");
			String server = Configuration.FTP_SERVER;
			for (Phenomenon phenomenon : sensor.getPhenomenonList()) {
				// create SensorML document for every phenomenon
				String sensorML = RequestBuilder.createSingleSensorML(sensor, phenomenon);
				// upload auf den FTP-Server
				String resourceName = phenomenon.getName() + "_" + System.currentTimeMillis() + ".xml";
				String descriptionURL = "ftp://" + server + "/" + resourceName;
				Utils.ftpUpload(server, Configuration.FTP_USER, Configuration.FTP_PASSWORD, resourceName, sensorML);
				// set URL to the sensors
				phenomenon.setSensorUrl(descriptionURL);
				// send connectSensor
				IMessage message = new ConnectSensorMessage(descriptionURL);
				sendMessage(message);
			}
			// position
			Phenomenon phenomenon = new Phenomenon("Position",
					"urn:ogc:def:phenomenon:OGC:1.0.30:position",
					null,
					ValueTypes.positionType);
			String sensorML = RequestBuilder.createSingleSensorML(sensor, phenomenon);
			String randomID = phenomenon.getName() + "_" + System.currentTimeMillis();
			String resourceName = randomID + ".xml";
			String descriptionURL = "ftp://" + server + "/" + resourceName;
			Utils.ftpUpload(server, Configuration.FTP_USER, Configuration.FTP_PASSWORD, resourceName, sensorML);
			sensor.setServiceDescriptionURL(descriptionURL);
			// send connectSensor
			IMessage message = new ConnectSensorMessage(descriptionURL);
			sendMessage(message);
		} catch (XMPPException e) {
			e.printStackTrace();
		}
	}

	public boolean insertObservation(Observation observation) {
		try {
			// connect to sensorBus
			connect();
			for (Phenomenon phenomenon : observation.getSensor().getPhenomenonList()) {
				// create PublishDataMessage
				String sensorDescriptionURL = phenomenon.getSensorUrl();
				String timeStamp = Utils.getSosConformTimeString(observation.getTime());
				String advertisedOutput = phenomenon.getDefinition();
				String locationTag = observation.getSensor().getStartPointName();
				String dataValue = observation.getObsProps_value().get(phenomenon.getName()).toString();
				IMessage message = new PublishDataMessage(sensorDescriptionURL, timeStamp, locationTag, advertisedOutput, dataValue);
				// send message
				sendMessage(message);
			}
			String sensorDescriptionURL = observation.getSensor().getServiceDescriptionURL();
			String timeStamp = Utils.getSosConformTimeString(observation.getTime());
			String advertisedOutput = "urn:ogc:def:phenomenon:OGC:1.0.30:position";
			String locationTag = observation.getSensor().getStartPointName();
			String dataValue = Utils.getSosConformPointString(observation.getPoint());
			// set point
			IMessage message = new PublishDataMessage(sensorDescriptionURL, timeStamp, locationTag, advertisedOutput, dataValue);
			sendMessage(message);
		} catch (XMPPException e) {
			e.printStackTrace();
		}
		// always true, because no response from sensor bus
		return true;
	}

	public String getConnectionType() {
		return "SensorBus";
	}

	public void stop() {
		if (conn.isConnected()) {
			conn.disconnect();
		}
	}

	private void connect() throws XMPPException {
		if (this.conn != null && this.conn.isConnected()) {
			Log.i(TAG, "XMPPConnector already connected!");
		} else {
			// needed for some older servers
			SASLAuthentication.supportSASLMechanism("PLAIN", 0);
			this.conn = new XMPPConnection(this.hostname);
			this.conn.connect();
			try {
				this.conn.login(this.login, this.passwort);
			} catch (Exception e) {
				Log.e(TAG, "Error while trying to log in: " + e.getMessage());
			}
		}
		// connect to management channel
		joinChannel(this.channel);
	}

	private boolean joinChannel(String channel) {
		this.chat = new MultiUserChat(this.conn, "sensorbus@conference.pluto001");
		try {
			this.chat.join(this.login);
		} catch (XMPPException e) {
			Log.e(TAG, e.getMessage());
			return false;
		}
		Log.i(TAG, "Joined chat: " + chat.getRoom() + " as " + chat.getNickname() + " about " + chat.getSubject());
		return true;
	}

	private void sendMessage(IMessage message) throws XMPPException {
		// create message
		String msgString = message.createMessage();
		// send message
		chat.sendMessage(msgString);
	}

}
