/**
 * Copyright 2012 52North Initiative for Geospatial Open Source Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.n52.android.tracker.encoder;

import org.n52.android.tracker.Constants;
import org.n52.android.tracker.Utils;
import org.n52.android.tracker.datastructure.Observation;
import org.n52.android.tracker.datastructure.Phenomenon;
import org.n52.android.tracker.datastructure.Sensor;

/**
 * Builds the xml request for the communication with the SOS service.
 * 
 * @author Jan Schulte
 * 
 */
public class RequestBuilder {

	public static String createRegisterSensor(Sensor sensor) {
		String request = Templates.getRegisterSensorTemplate();
		// procedure ID
		request = request.replace(Constants.PROCEDUREID, sensor.getProcedureID());
		// StartPoint
		request = request.replace(Constants.STARTPOINT, sensor.getStartPointName());
		// Longitude
		request = request.replace(Constants.LONGITUDE, sensor.getPoint().getLongitude() + "");
		// Latitude
		request = request.replace(Constants.LATITUDE, sensor.getPoint().getLatitude() + "");
		// Altitude
		request = request.replace(Constants.ALTITUDE, sensor.getPoint().getAltitude() + "");
		// InputList and OutputList
		if (sensor.getPhenomenonList() != null || sensor.getPhenomenonList().isEmpty()) {
			StringBuffer inputList = new StringBuffer();
			StringBuffer outputList = new StringBuffer();
			for (Phenomenon phen : sensor.getPhenomenonList()) {
				// create input
				inputList.append(createInput(phen));
				// create output
				outputList.append(createOutput(phen, sensor));
			}
			request = request.replace(Constants.INPUTLIST, inputList.toString());
			request = request.replace(Constants.OUTPUTLIST, outputList.toString());
		} else {
			request = request.replace(Constants.INPUTLIST, "");
			request = request.replace(Constants.OUTPUTLIST, "");
		}
		// Offering
		request = request.replace(Constants.OFFERING, sensor.getOffering());
		request = request.replace(Constants.OFFERINGBESCHREIBUNG, sensor.getOfferingDescription());

		return request;
	}

	public static String createSingleSensorML(Sensor sensor, Phenomenon phenomenon) {
		String singSens = Templates.getSingleSensor();
		// procedure ID
		singSens = singSens.replace(Constants.PROCEDUREID, sensor.getProcedureID() + "_" + phenomenon.getName());
		// startpoint
		singSens = singSens.replace(Constants.STARTPOINT, sensor.getStartPointName());
		// longitude
		singSens = singSens.replace(Constants.LONGITUDE, sensor.getPoint().getLongitude() + "");
		// latitude
		singSens = singSens.replace(Constants.LATITUDE, sensor.getPoint().getLatitude() + "");
		// altitude
		singSens = singSens.replace(Constants.ALTITUDE, sensor.getPoint().getAltitude() + "");
		// input
		singSens = singSens.replace(Constants.INPUTLIST, createInput(phenomenon));
		// output
		singSens = singSens.replace(Constants.OUTPUTLIST, createOutput(phenomenon, sensor));
		return singSens;
	}

	// Resources resources;

	public static String createInsertObservation(Observation observation, boolean template) {
		String request = Templates.getInsertObservationTemplate();
		// procedure ID
		request = request.replaceAll(Constants.PROCEDUREID, observation.getSensor().getProcedureID());
		// begin
		request = request.replace(Constants.BEGIN, Utils.getSosConformTimeString(observation.getTime()));
		// end
		request = request.replace(Constants.END, Utils.getSosConformTimeString(observation.getTime()));
		// longitude
		request = request.replace(Constants.LONGITUDE, observation.getSensor().getPoint().getLongitude() + "");
		// latitude
		request = request.replace(Constants.LATITUDE, observation.getSensor().getPoint().getLatitude() + "");
		// Components and Fields
		if (observation.getSensor().getPhenomenonList() != null
				|| observation.getSensor().getPhenomenonList().isEmpty()) {
			StringBuffer components = new StringBuffer();
			StringBuffer fields = new StringBuffer();
			for (Phenomenon phen : observation.getSensor().getPhenomenonList()) {
				// add component
				String component = Templates.getComponent();
				component = component.replace(Constants.COMPONENT_DEFINITION, phen.getDefinition());
				components.append(component);
				// add field
				String field = null;
				if (phen.getValueType().equals(Constants.ValueTypes.booleanType)) {
					field = Templates.getFieldBoolean();
				} else if (phen.getValueType().equals(Constants.ValueTypes.countType)) {
					field = Templates.getFieldCount();
				} else if (phen.getValueType().equals(Constants.ValueTypes.numericType)) {
					field = Templates.getFieldQuantity();
					field = field.replace(Constants.UOM, phen.getUnit());
				} else if (phen.getValueType().equals(Constants.ValueTypes.textType)) {
					field = Templates.getFieldText();
				}
				field = field.replace(Constants.FIELD_NAME, phen.getName());
				field = field.replace(Constants.FIELD_DEFINITION, phen.getDefinition());
				fields.append(field);
			}
			request = request.replace(Constants.COMPONENTS, components.toString());
			request = request.replace(Constants.FIELDS, fields.toString());
		} else {
			request = request.replace(Constants.COMPONENTS, "");
			request = request.replace(Constants.FIELDS, "");
		}
		// Gml ID
		request = request.replace(Constants.GMLID, observation.getSensor().getFoiID());
		// Count
		request = request.replace(Constants.COUNT, "1");
		// Decimal seperator
		request = request.replace(Constants.DECIMALSEP, Constants.DECIMALSEP_SIGN);
		// Token seperator
		request = request.replace(Constants.TOKENSEP, Constants.TOKENSEP_SIGN);
		// Block seperator
		request = request.replace(Constants.BLOCKSEP, Constants.BLOCKSEP_SIGN);
		// tamplate or values
		if (!template) {
			request = request.replace(Constants.VALUES, createValues(observation));
		} else {
			request = request.replace(Constants.VALUES, "resultTemplate");
		}
		return request;
	}

	public static String createInsertResult(Observation observation, String templateID) {
		String request = Templates.getInsertResult();
		// template_id
		request = request.replace(Constants.TEMPLATE_ID, templateID);
		// values
		request = request.replace(Constants.VALUES, createValues(observation));
		return request;
	}

	private static String createOutput(Phenomenon phenomenon, Sensor sensor) {
		String output = null;
		if (phenomenon.getValueType().equals(Constants.ValueTypes.booleanType)) {
			output = Templates.getOutputBoolean();
		} else if (phenomenon.getValueType().equals(Constants.ValueTypes.countType)) {
			output = Templates.getOutputCount();
		} else if (phenomenon.getValueType().equals(Constants.ValueTypes.numericType)) {
			output = Templates.getOutputQuantity();
			output = output.replace(Constants.UOM, phenomenon.getUnit());
		} else if (phenomenon.getValueType().equals(Constants.ValueTypes.textType)) {
			output = Templates.getOutputText();
		} else if (phenomenon.getValueType().equals(Constants.ValueTypes.positionType)) {
			output = Templates.getOutputPosition();
		}
		output = output.replace(Constants.OUTPUT_NAME, phenomenon.getName());
		output = output.replace(Constants.OUTPUT_DEFINITION, phenomenon.getDefinition());
		output = output.replace(Constants.OFFERING, sensor.getOffering());
		output = output.replace(Constants.OFFERINGBESCHREIBUNG, sensor.getOfferingDescription());
		return output;
	}

	private static String createInput(Phenomenon phenomenon) {
		String input = Templates.getInput();
		input = input.replace(Constants.INPUT_NAME, phenomenon.getName());
		input = input.replace(Constants.INPUT_DEFINITION, phenomenon.getDefinition());
		return input;
	}

	private static String createValues(Observation observation) {
		// Values
		StringBuffer values = new StringBuffer();
		// time
		values.append(Utils.getSosConformTimeString(observation.getTime()));
		values.append(Constants.TOKENSEP_SIGN);
		// foi ID
		values.append(observation.getSensor().getFoiID());
		values.append(Constants.TOKENSEP_SIGN);
		// point
		values.append(Utils.getSosConformPointString(observation.getPoint()));
		values.append(Constants.TOKENSEP_SIGN);
		// phenomenon values
		if (observation.getSensor().getPhenomenonList() != null
				|| observation.getSensor().getPhenomenonList().isEmpty()) {
			for (Phenomenon phen : observation.getSensor().getPhenomenonList()) {
				values.append(observation.getObsProps_value().get(phen.getName()));
				values.append(Constants.TOKENSEP_SIGN);
			}
		}
		values.deleteCharAt(values.length() - 1);
		values.append(Constants.BLOCKSEP_SIGN);

		return values.toString();
	}

}
