/**
 * Copyright 2012 52North Initiative for Geospatial Open Source Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.n52.android.tracker.services.impl;

import java.util.HashMap;
import java.util.Map;
import java.util.Stack;

import org.n52.android.tracker.R;
import org.n52.android.tracker.Utils;
import org.n52.android.tracker.activities.Data;
import org.n52.android.tracker.activities.Logging;
import org.n52.android.tracker.activities.Preferences;
import org.n52.android.tracker.activities.Tracker;
import org.n52.android.tracker.connection.IConnection;
import org.n52.android.tracker.datastructure.Observation;
import org.n52.android.tracker.datastructure.Phenomenon;
import org.n52.android.tracker.datastructure.Point;
import org.n52.android.tracker.datastructure.Sensor;
import org.n52.android.tracker.services.ITrackerServiceBinder;

import android.app.Notification;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Binder;
import android.os.Build;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;
import android.widget.Toast;

/**
 * This service gets the location by the LocationListener and puts the location
 * together with possible other measurements on a stack. Furthermore a Thread is
 * started which creates observations out of the stack and sends this
 * observations to the SOS service.
 * 
 * @author Jan Schulte
 */
public class TrackerService extends Service {

	private static final String TAG = "TrackerService";

	public static final String ACTION_LOCATION = "de.schulte.intent.newlocationevent";

	public static boolean tracking = false;

	private final IBinder trackerServiceBinder = new TrackerServiceBinder();

	private Sensor trackedSensor;

	private LocationManager locManager;

	private RequestManagerThread requestManager;

	private Stack<Observation> obsStack = new Stack<Observation>();

	private LocationListener locListener = new LocationListener() {

		public void onStatusChanged(String provider, int status, Bundle extras) {
			//
		}

		public void onProviderEnabled(String provider) {
			Logging.addEntry("LocationListener:onProviderEnabled");
		}

		public void onProviderDisabled(String provider) {
			if (provider == "gps") {
				Toast.makeText(getApplicationContext(),
						"Please enable GPS, to have a precise position!",
						Toast.LENGTH_LONG).show();
				Logging.addEntry("Please enable GPS, to have a precise position!");
			}
		}

		public void onLocationChanged(Location location) {
			if (tracking) {
				// for testing
				Map<String, Object> obsProps_value = new HashMap<String, Object>();
				for (Phenomenon phen : trackedSensor.getPhenomenonList()) {
					// speed
					if (phen.getName().equals("Speed")) {
						obsProps_value.put(phen.getName(), location.getSpeed() * 3.6);
					}
					// provider
					else if (phen.getName().equals("Provider")) {
						obsProps_value.put(phen.getName(), location.getProvider());
					}
					// accuracy
					else if (phen.getName().equals("Accuracy")) {
						obsProps_value.put(phen.getName(), location.getAccuracy());
					}
					// altitude
					else if (phen.getName().equals("Altitude")) {
						obsProps_value.put(phen.getName(), location.getAltitude());
					} else {
						obsProps_value.put(phen.getName(), Utils.generateRandomValue(phen.getValueType()));
					}
				}

				// workaround for the real time in the emulator
				long time;
				if ("google_sdk".equals(Build.PRODUCT)) {
					time = System.currentTimeMillis();
				} else {
					time = location.getTime();
				}

				Observation observation = new Observation(trackedSensor, time, new Point(location.getLongitude(),
						location.getLatitude(),
						location.getAltitude()), obsProps_value);
				observation.setLatencyTime(System.currentTimeMillis());
				obsStack.push(observation);
			}
			// send location intent
			Intent locIntent = new Intent(ACTION_LOCATION);
			locIntent.putExtra("location", location);
			Logging.addLocation(location);
			Data.lastLoc = location;
			sendBroadcast(locIntent);
		}
	};

	@Override
	public void onCreate() {
		Log.d(TAG, "onCreate(): entered...");
		Long minTime = Long.valueOf(Preferences.getPreferences(this).getString(this
				.getString(R.string.prefs_key_minseconds),
				this.getString(R.string.prefs_default_minseconds))) * 1000;
		Float minDistance = Float.valueOf(Preferences.getPreferences(this).getString(this
				.getString(R.string.prefs_key_minmeters),
				this.getString(R.string.prefs_default_minmeters)));
		locManager = (LocationManager) this.getSystemService(LOCATION_SERVICE);
		locManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, minTime, minDistance, locListener);
		locManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, minTime, minDistance, locListener);
		super.onCreate();
	}

	@Override
	public IBinder onBind(Intent intent) {
		Log.d(TAG, "onBind(): entered...");
		return trackerServiceBinder;
	}

	@Override
	public void onDestroy() {
		if (!tracking) {
			locManager.removeUpdates(locListener);
			super.onDestroy();
		}
	}

	public class TrackerServiceBinder extends Binder implements ITrackerServiceBinder {

		public void startTrack(Sensor sensor, long minTime, float minDistance, IConnection connection) {
			if (!tracking) {
				tracking = true;
				trackedSensor = sensor;
				String info = "Start the tracking by connection type: " + connection.getConnectionType() + "!";
				Logging.addEntry(info);
				Toast.makeText(getApplicationContext(), info, Toast.LENGTH_LONG).show();

				// foreground tracker service
				startTrackerForeground();

				// thread manages the requests
				requestManager = new RequestManagerThread(connection, sensor, obsStack);
				requestManager.start();
			}
		}

		public void stopTrack() {
			if (tracking) {
				Logging.addEntry("Stop Tracking!");
				Toast.makeText(getApplicationContext(), "Stop Tracking!", Toast.LENGTH_LONG).show();
				// stop tracking
				requestManager.setTracking(false);
				// stop foreground tracker
				stopTrackerForeground();
				tracking = false;
			}
		}
	}

	private void startTrackerForeground() {
		Notification notification = new Notification(R.drawable.app_icon,
				getText(R.string.notification_title),
				System.currentTimeMillis());
		PendingIntent contentIntent = PendingIntent.getActivity(this, 0, new Intent(this, Tracker.class), 0);
		notification.setLatestEventInfo(this,
				getText(R.string.notification_content_title),
				getText(R.string.notification_content_text),
				contentIntent);
		startForeground(12345678, notification);
	}

	private void stopTrackerForeground() {
		stopForeground(true);
	}

}